
# generic methods for "MosaicsPeak" class

setMethod(
    f="show",
    signature="MosaicsPeak",
    definition=function( object ) {  
              
        peakParam <- object@peakParam
        
        analysisType <- peakParam@analysisType
        signalModel <- peakParam@signalModel
        FDR <- peakParam@FDR
        maxgap <- peakParam@maxgap
        minsize <- peakParam@minsize
        thres <- peakParam@thres
        
        cat( "Summary: MOSAiCS peak calling (class: MosaicsPeak)\n" )
        cat( "--------------------------------------------------\n" )
        cat( "final model: " )
        switch( analysisType,
            "OS" = {
                cat( "one-sample analysis " )
            },
            "TS" = {
                cat( "two-sample analysis (with M & GC) " )
            },
            "IO" = {
                cat( "two-sample analysis (input only) " )
            }
        )
        switch( signalModel,
            "1S" = {
                cat( "with one signal component\n" )
            },
            "2S" = {
                cat( "with two signal components\n" )
            }
        )        
        cat( "setting: FDR = ", FDR, ", maxgap = ", maxgap,
            ", minsize = ", minsize, ", thres = ", thres, "\n", sep="" )
        if ( length( object@peakList$peakStart ) > 0 ) {
		        cat( "# of peaks = ", nrow(object@peakList), "\n", sep="" )
		        cat( "median peak length = ", median(object@peakList$peakSize), "\n", sep="" )
		        cat( "empirical FDR = ", round(empFDR(object)*10000)/10000, "\n", sep="" )
		    } else {
		        # exception handling (no peak case)
						
						cat( "(no peak detected)\n" )
		    }
        cat( "--------------------------------------------------\n" )
    }
)

setMethod(
    f="print",
    signature="MosaicsPeak",
    definition=function( x ) {    
        peakList <- x@peakList
        
        if ( nrow(peakList) > 0 ) {
		        printForm <- x@peakList
		        return(printForm)
		    } else {
		        # exception handling (no peak case)
		        
		        cat( "Info: no peak detected. Nothing can be returned.\n" )		    
		    }
    }
)

setMethod(
    f="export",
    signature="MosaicsPeak",
    definition=function( object, type=NA, fileLoc=NA, fileName=NA, chrID=NA ) {
        # error treatment: check invalid type
        
        if ( is.na(type) )
        {
            cat( "Warning: 'type' is not specified by the user.\n" )
            cat( "Warning: 'type' is specified as 'txt' instead.\n" )
            type <- "txt"        
        }
        
        allType <- c("txt","gff","bed")
        invalidType <- TRUE
        for ( i in 1:length(type) )
        {
            if ( length(which(!is.na(match(type[i],allType))))==0 )
            {
                invalidType <- FALSE
            }
        }
        if ( !invalidType )
        {
            cat( "Warning: 'type' incorrect.\n" )
            cat( "Warning: 'type' is specified as 'txt' instead.\n" )
            type <- "txt"
        }
        
        # error treatment: 'fileLoc' not specified
        
        if ( is.na(fileLoc) )
        {
            cat( "Warning: 'fileLoc' is not specified by the user.\n" )
            cat( "Warning: 'fileLoc' is specified as current directory instead.\n" )
            fileLoc <- getwd()
            cat( "Warning: current directory = ", fileLoc, "\n" )
        }
        
        # error treatment: 'fileName' not specified
        
        if ( is.na(fileName) )
        {
            cat( "Warning: 'fileName' is not specified by the user.\n" )
            cat( "Warning: 'fileName' is specified as 'peakList' instead.\n" )
            fileName <- paste("peakList.",type,sep="")
        }
        
        # error treatment: 'chrID' not specified
        
        if ( is.na(chrID) )
        {
            cat( "Warning: 'chrID' is not specified by the user.\n" )
            cat( "Warning: 'chrID' is specified as 'chrNA' instead.\n" )
            chrID <- "chrNA"
        }
        
        # export peak lists
        
        peakList <- object@peakList
        
        if ( nrow(object@peakList) > 0 ) {        
		        cat( "Info: exporting the peak list...\n" )
		        switch( type,
		            "gff" = {
		                .exportGFF( peakList=peakList, fileLoc=fileLoc, fileName=fileName, chrID=chrID )
		            },
		            "bed" = {
		                .exportBED( peakList=peakList, fileLoc=fileLoc, fileName=fileName, chrID=chrID )
		            },
		            "txt" = {
		                .exportTXT( peakList=peakList, fileLoc=fileLoc, fileName=fileName, chrID=chrID )
		            }
		        )
		    } else {
		        # exception handling (no peak case)
		        
		        cat( "Info: no peak detected. Nothing exported.\n" )
		    }
    }
)

setMethod(
    f="empFDR",
    signature="MosaicsPeak",
    definition=function( object ) {
        return(object@empFDR)
    }
)

setMethod(
    f="bdBin",
    signature="MosaicsPeak",
    definition=function( object ) {
        return(object@bdBin)
    }
)
