\name{BinData-class}
\Rdversion{1.1}
\docType{class}
\alias{BinData-class}
\alias{coord,BinData-method}
\alias{gcContent,BinData-method}
\alias{input,BinData-method}
\alias{mappability,BinData-method}
%\alias{mosaicsFit,BinData-method}
\alias{plot,BinData,missing-method}
\alias{print,BinData-method}
\alias{show,BinData-method}
\alias{tagCount,BinData-method}

\title{Class "BinData" }
\description{
This class represents bin-level ChIP-seq data.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("BinData", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{coord}:}{Object of class \code{"numeric"}, 
        a vector of coordinates. }
    \item{\code{tagCount}:}{Object of class \code{"numeric"}, 
        a vector of tag counts of ChIP sample. }
    \item{\code{mappability}:}{Object of class \code{"numeric"}, 
        a vector of mappability score. }
    \item{\code{gcContent}:}{Object of class \code{"numeric"}, 
        a vector of GC content score. }
    \item{\code{input}:}{Object of class \code{"numeric"}, 
        a vector of tag counts of matched control sample (input sample). }
    \item{\code{dataType}:}{Object of class \code{"character"}, 
        indicating how tags were processed. Possible values are 
        "unique" (only uniquely aligned tags were retained)
        and "multi" (tags aligned to multiple locations were also retained). }
  }
}
\section{Methods}{
  \describe{
    \item{coord}{\code{signature(object = "BinData")}: return a vector of coordinates. }
    \item{gcContent}{\code{signature(object = "BinData")}: return a vector of GC content score. }
    \item{input}{\code{signature(object = "BinData")}: return a vector of tag counts of input sample. }
    \item{mappability}{\code{signature(object = "BinData")}: return a vector of mappability score. }
    \item{mosaicsFit}{\code{signature(object = "BinData")}: 
        fit MOSAiCS model from a bin-level ChIP-seq data. }
    \item{plot}{\code{signature(x = "BinData", y = "missing", plotType = NULL )}: 
        provide exploratory plots of mean ChIP tag counts.
        This method plots mean ChIP tag counts versus mappability score, GC content score,
				and input tag counts, with 95$\%$ confidence interval,  
				for \code{plotType="M"}, \code{plotType="GC"}, and \code{plotType="input"}, respectively.
        \code{plotType="M|input"} and \code{plotType="GC|input"} provide
        plots of mean ChIP tag counts versus mappability and GC content score, respectively,
        conditional on input tag count. 
        If \code{plotType} is not specified, this method plots histogram of mean ChIP tag counts. }
    \item{print}{\code{signature(x = "BinData")}: 
        return bin-level data in data frame format. }
    \item{show}{\code{signature(object = "BinData")}: provide brief summary of the object. }
    \item{tagCount}{\code{signature(object = "BinData")}: return a vector of tag counts of ChIP sample. }
     }
}
\references{
Kuan, PF, D Chung, JA Thomson, R Stewart, and S Keles (2010), "A Statistical Framework
for the Analysis of ChIP-Seq Data", submitted (\url{http://works.bepress.com/sunduz_keles/19/}).
}
\author{ Dongjun Chung, Pei Fen Kuan, Sunduz Keles }
\seealso{
\code{\link{readBins}}, \code{\link{mosaicsFit}}.
}
\examples{
showClass("BinData")
\dontrun{
data(bin1)
plot(bin1)
plot( bin1, plotType="M" )
plot( bin1, plotType="GC" )
}
}
\keyword{classes}
