\name{export}
\alias{export}
\alias{export,MosaicsPeak-method}
\title{
Export peak calling results to text files
}
\description{
Export peak calling results to text files in TXT, BED, or GFF file format.
}
\usage{
export(object, ...)
\S4method{export}{MosaicsPeak}( object, type=NA, fileLoc=NA, fileName=NA, chrID=NA )
}
\arguments{
    \item{object}{Object of class \code{MosaicsPeak},
        peak calling results obtained using method \code{mosaicsPeak}. }
    \item{type}{File format.
        Possible values are \code{"txt"}, \code{"bed"}, and \code{"gff"}. See Details. }
    \item{fileLoc}{Directory of the exported file. }    
    \item{fileName}{Name of the exported file. }     
    \item{chrID}{Chromosome ID, which is included in the exported file,
        e.g. \code{chrID="chr22"}. }  
    \item{...}{ Other parameters to be passed through to generic \code{export}.}
}
\details{
TXT file format (\code{type="txt"}) exports peak calling results in the most informative way.
Columns include peak start position, peak stop position, peak size, summed tag counts in each peak,
averaged p-value, and minimum p-value.
\code{type="bed"} and \code{type="gff"} export peak calling results in standard BED and GFF file format,
respectively, where score is summed tag counts in each peak.
If no peak is detected, files are not exported.
}
\references{
Kuan, PF, D Chung, JA Thomson, R Stewart, and S Keles (2010), "A Statistical Framework
for the Analysis of ChIP-Seq Data", submitted (\url{http://works.bepress.com/sunduz_keles/19/}).
}
\author{ Dongjun Chung, Pei Fen Kuan, Sunduz Keles }
\seealso{
\code{\link{mosaicsPeak}}, \code{\linkS4class{MosaicsPeak}}.
}
\examples{
\dontrun{
data(fit1)
peak1 <- mosaicsPeak( fit1, signalModel = "2S", FDR = 0.05 )
export( peak1, type = "txt", fileLoc = "./", fileName = "OSpeakList.txt", chrID = "chr22" )
export( peak1, type = "bed", fileLoc = "./", fileName = "OSpeakList.bed", chrID = "chr22" )
export( peak1, type = "gff", fileLoc = "./", fileName = "OSpeakList.gff", chrID = "chr22" )
}
}
\keyword{models}
\keyword{methods}
