\name{mosaics-package}
\alias{mosaics-package}
\alias{mosaics}
\docType{package}
\title{
MOSAiCS (MOdel-based one and two Sample Analysis and Inference for ChIP-Seq)
}
\description{
This package provides functions for fitting MOSAiCS,
a statistical framework to analyze one-sample or two-sample ChIP-seq data.
}
\details{
\tabular{ll}{
Package: \tab mosaics\cr
Type: \tab Package\cr
Version: \tab 0.0.9\cr
Date: \tab 2010-09-24\cr
License: \tab GPL (>= 2)\cr
LazyLoad: \tab yes\cr
}
This package contains three main classes, \code{BinData}, \code{MosaicsFit}, and \code{MosaicsPeak},
which represent bin-level data, MOSAiCS model fit, and MOSAiCS peak calling results, respectively.
This package contains three main methods,\code{readBins}, \code{mosaicsFit}, and \code{mosaicsPeak}.
\code{readBins} method imports bin-level data and construct \code{BinData} class object.
\code{mosaicsFit} method fits MOSAiCS model using \code{BinData} class object and
 constructs \code{MosaicsFit} class object.
\code{mosaicsPeak} method calls peaks using \code{MosaicsFit} class object and
 construct \code{MosaicsPeak} class object.
\code{MosaicsPeak} class object can be exported as text files or transformed into data frame
and can be used the downstream analysis.
This package also provides methods for simple data exploratory analysis.
}
\author{
Dongjun Chung, Pei Fen Kuan, Sunduz Keles

Maintainer: Dongjun Chung <chungdon@stat.wisc.edu>
}
\references{
Kuan, PF, D Chung, JA Thomson, R Stewart, and S Keles (2010), "A Statistical Framework
for the Analysis of ChIP-Seq Data", submitted (\url{http://works.bepress.com/sunduz_keles/19/}).
}
\keyword{ package }
\seealso{
\code{\link{readBins}}, \code{\link{mosaicsFit}}, \code{\link{mosaicsPeak}} ,
\code{\linkS4class{BinData}}, \code{\linkS4class{MosaicsFit}}, \code{\linkS4class{MosaicsPeak}}.
}
\examples{
\dontrun{
bin1 <- readBins(type = c("chip", "M", "GC", "N"), 
 fileName = c("./chip_chr22.txt","./M_chr22.txt", "./GC_chr22.txt", "./N_chr22.txt"))
data(bin1)
bin1
print(bin1)[1:10, ]
plot(bin1)
fit1 <- mosaicsFit( bin1, analysisType="OS" )
data(fit1)
fit1
plot(fit1)
peak1 <- mosaicsPeak(fit1, signalModel = "2S", FDR = 0.05 )
peak1
print(peak1)[1:10, ]
export(peak1, type = "txt", fileLoc = "./", fileName = "OSpeakList.txt", chrID = "chr22")
export(peak1, type = "bed", fileLoc = "./", fileName = "OSpeakList.bed", chrID = "chr22")
export(peak1, type = "gff", fileLoc = "./", fileName = "OSpeakList.gff", chrID = "chr22")
}
}
