\name{mosaicsPeak}
\alias{mosaicsPeak}
\alias{mosaicsPeak,MosaicsFit-method}
\title{
Call peaks based on fitted MOSAiCS model
}
\description{
Call peaks using \code{MosaicsFit} class object, which is a fitted MOSAiCS model.
}
\usage{
mosaicsPeak( object, ... )
\S4method{mosaicsPeak}{MosaicsFit}( object, signalModel="2S", FDR=0.05, maxgap=200, minsize=50, thres=10 )
}
%- maybe also 'usage' for other objects documented here.
\arguments{
    \item{object}{Object of class \code{MosaicsFit},
        fitted MOSAiCS model obtained using function \code{mosaicsFit}. }
    \item{signalModel}{Signal model.
        Possible values are "1S" (one-signal-component model) and
        "2S" (two-signal-component model). Default is "2S". }
    \item{FDR}{False discovery rate. Default is 0.05. }    
    \item{maxgap}{Distance (in bp) to combine initial peaks. Default is 200 (bp). }     
    \item{minsize}{Minimum size (in bp) of the initial peak so that it is claimed as a peak. Default is 50 (bp). }
    \item{thres}{Minimum ChIP tag count in each bin so that the bin is claimed as a peak. Default is 10 (tag count should be larger than 10). }  
    \item{...}{ Other parameters to be passed through to generic \code{mosaicsPeak}.}
}
\details{
When peaks are called, signal model for the final model needs to be specified.
The optimal choice of the number of signal components depends on the characteristics of ChIP-seq data.
In order to support user in determination of optimal signal model, 
Bayesian Information Criterion (BIC) values and Goodness of Fit (GOF) plot are provided.
These can be obtained by applying \code{show} and \code{plot} methods
to the \code{MosaicsFit} class object, which is a fitted MOSAiCS model.
}
\value{
Construct \code{MosaicsPeak} class object.
}
\references{
Kuan, PF, D Chung, JA Thomson, R Stewart, and S Keles (2010), "A Statistical Framework
for the Analysis of ChIP-Seq Data", submitted (\url{http://works.bepress.com/sunduz_keles/19/}).
}
\author{ Dongjun Chung, Pei Fen Kuan, Sunduz Keles }
\seealso{
\code{\link{mosaicsFit}} , \code{\linkS4class{MosaicsPeak}}, \code{\linkS4class{MosaicsFit}}.
}
\examples{
data(fit1)
peak1 <- mosaicsPeak( fit1, signalModel = "2S", FDR = 0.05 )
peak1
print(peak1)[1:10, ]
export(peak1, type = "txt", fileLoc = "./", fileName = "OSpeakList.txt", chrID = "chr22")
export(peak1, type = "bed", fileLoc = "./", fileName = "OSpeakList.bed", chrID = "chr22")
export(peak1, type = "gff", fileLoc = "./", fileName = "OSpeakList.gff", chrID = "chr22")
}
\keyword{models}
\keyword{methods}
