\name{readBins}
\alias{readBins}
\title{
Import bin-level ChIP-sep data
}
\description{
Import and pre-process 
all or subset of bin-level ChIP-sep data, including ChIP data, input data, 
mappability score, GC content score, and sequence ambiguity score.
}
\usage{
readBins( type = c("chip", "M", "GC", "N"), fileName = NULL,
    dataType = "unique", rounding = 100 )
}
\arguments{
  \item{type}{
    Character vector indicating data types to be imported.
    This vector can contain \code{"chip"} (ChIP data), \code{"input"} (input data), 
    \code{"M"} (mappability score), \code{"GC"} (GC content score), and \code{"N"} (sequence ambiguity score).
    Currently, \code{readBins} permits only the following combinations:
		\code{c("chip", "input", "M", "GC", "N")}, \code{c("chip", "M", "GC", "N")},  
		\code{c("chip", "input", "N")}, and \code{c("chip", "input")}.
    Default is \code{c("chip", "M", "GC", "N")}.
}
  \item{fileName}{
    Character vector of file names, each of which matches each element of \code{type}.
    \code{type} and \code{fileName} should have the same length and
    corresponding elements in two vectors should appear in the same order.
}
  \item{dataType}{
    How tags were processed? Possible values are 
    either \code{"unique"} (only uniquely aligned tags were retained)
    or \code{"multi"} (tags aligned to multiple locations were also retained).
}
  \item{rounding}{
    How are mappability score and GC content score rounded?
    Default is 100 and this indicates rounding of mappability score and GC content score
    to the nearest hundredth.
}
}
\details{
The imported data type constraints the analysis that can be implemented.
If \code{type=c("chip", "M", "GC", "N")}, only one-sample analysis is permitted.
If \code{type=c("chip", "input")} or \code{c("chip", "input", "N")}, 
only two-sample analysis without using mappability and GC content is possible. 
For \code{type=c("chip", "input", "M", "GC", "N")},
user can do all the one- or two-sample analysis. See also help page of \code{mosaicsFit}.

Bin-level ChIP and input data can be obtained from the aligned data (e.g., a file
obtained from ELAND) using the perl codes we provide. Users can download these perl codes from
\url{http://groups.google.com/group/mosaics_user_group}. For human genome (HG18) and mouse
genome (MM9), we provide mappability score, GC content score, and sequence ambiguity score
for each chromosome. Users can download these files from \url{http://groups.google.com/group/
mosaics_user_group}. This webpage and the vignette provide further details.
}
\value{
Construct 'MosaicsFit' class object.
}
\references{
Kuan, PF, D Chung, JA Thomson, R Stewart, and S Keles (2010), "A Statistical Framework
for the Analysis of ChIP-Seq Data", submitted (\url{http://works.bepress.com/sunduz_keles/19/}).
}
\author{ Dongjun Chung, Pei Fen Kuan, Sunduz Keles }
\seealso{
\code{\link{mosaicsFit}}, \code{\linkS4class{BinData}}.
}
\examples{
\dontrun{
bin1 <- readBins(type = c("chip", "M", "GC", "N"), 
 fileName = c("./chip_chr22.txt","./M_chr22.txt", "./GC_chr22.txt", "./N_chr22.txt"))
data(bin1)
bin1
print(bin1)[1:10, ]
plot(bin1)
}
}
\keyword{models}
\keyword{methods}
